// Package main is the application entrypoint.
package main

import (
	"fmt"
	"log"
	"os"
	"os/user"

	"github.com/gotk3/gotk3/glib"
	"github.com/gotk3/gotk3/gtk"
	"varia.zone/hui/internal/bldr"
	"varia.zone/hui/internal/conf"
	"varia.zone/hui/internal/model"
	"varia.zone/hui/internal/ui"
)

const appID = "zone.varia.hui"

func main() {
	application, err := gtk.ApplicationNew(appID, glib.APPLICATION_FLAGS_NONE)
	if err != nil {
		log.Fatal(fmt.Errorf("main: unable to initialise GTK application: %s", err))
	}

	usr, err := user.Current()
	if err != nil {
		log.Fatal(fmt.Errorf("main: unable to detect current user: %s", err))
	}

	m := &model.Model{
		User: usr,
	}

	application.Connect("activate", func() {
		builder, err := gtk.BuilderNewFromFile(conf.MainWindowGladeFile)
		if err != nil {
			err = fmt.Errorf("main: unable to load '%s' from file: %s", conf.MainWindowGladeFile, err)
			log.Fatal(err)
		}
		m.Builder = builder

		if _, err := os.Stat(conf.HugoBinPath(m.User.HomeDir)); err != nil && os.IsNotExist(err) {
			if err := ui.SwitchStackPage(m, conf.InitStackPage); err != nil {
				if err := ui.ShowErrDialog(m, err); err != nil {
					log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
				}
			}

			if err := ui.HideSiteButtons(m); err != nil {
				if err := ui.ShowErrDialog(m, err); err != nil {
					log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
				}
			}

		} else {
			if err := ui.SwitchStackPage(m, conf.EmptyStackPage); err != nil {
				if err := ui.ShowErrDialog(m, err); err != nil {
					log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
				}
			}

			if err := ui.PopulateOpenSiteMenu(m); err != nil {
				if err := ui.ShowErrDialog(m, err); err != nil {
					log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
				}
			}
		}

		signals := map[string]interface{}{
			"hideErrorDialog": func() {
				if err := ui.HideErrorDialog(m); err != nil {
					if err := ui.ShowErrDialog(m, err); err != nil {
						log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
					}
				}
			},
			"initialise": func() {
				if err := ui.Initialise(m); err != nil {
					if err := ui.ShowErrDialog(m, err); err != nil {
						log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
					}
				}
			},
			"showNewSiteDialog": func() {
				if err := ui.ShowNewSiteDialog(m); err != nil {
					if err := ui.ShowErrDialog(m, err); err != nil {
						log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
					}
				}
			},
			"hideNewSiteDialog": func() {
				if err := ui.HideNewSiteDialog(m); err != nil {
					if err := ui.ShowErrDialog(m, err); err != nil {
						log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
					}
				}
			},
			"createNewSite": func() {
				if err := ui.CreateNewSite(m); err != nil {
					if err := ui.ShowErrDialog(m, err); err != nil {
						log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
					}
				}
			},
			"previewSite": func() {
				if err := ui.PreviewSite(m); err != nil {
					if err := ui.ShowErrDialog(m, err); err != nil {
						log.Fatal(fmt.Errorf("main: unable to show error dialog: %s", err))
					}
				}
			},
		}

		m.Builder.ConnectSignals(signals)

		window, err := bldr.GetWindow(m.Builder, conf.MainWindow)
		if err != nil {
			log.Fatal(err)
		}

		window.Show()
		application.AddWindow(window)
	})

	os.Exit(application.Run(os.Args))
}
